from datetime import date

from django.apps import apps
from django.db import models
from ldapdb.models.fields import CharField, DateField, IntegerField, ListField
import ldapdb.models

from ticdesk_account.models import TeckidsLdapMixin

class Voucher(models.Model):
    code = models.CharField(max_length=8, unique=True)
    project_cn = models.CharField(max_length=50)
    discount = models.IntegerField(default=100)

    used = models.BooleanField(default=False)
    used_person_uid = models.CharField(max_length=20, blank=True, null=True)

# FIXME make inheritance from TeckidsGroup work
class TeckidsProject(ldapdb.models.Model, TeckidsLdapMixin):
    class Meta:
        managed = False
        app_label = 'ticdesk_events'
        ordering = ('date_event',)

    base_dn = 'ou=Groups,dc=teckids,dc=org'
    object_classes = ['posixGroup', 'groupOfNames', 'teckidsGroup', 'teckidsProject']

    # Group name details    
    cn = CharField(db_column='cn', max_length=50, primary_key=True)
    display_name = CharField(db_column='displayName', max_length=100)
    description = CharField(db_column='description', max_length=500)

    # Members of this group
    members = ListField(db_column='member')
    member_uids = ListField(db_column='memberUid')
    owners = ListField(db_column='owner')

    l = CharField(db_column='l', max_length=50)

    date_event = DateField(db_column='teckidsProjectDate')
    date_registration = DateField(db_column='teckidsProjectRegistrationDeadline')
    date_retraction = DateField(db_column='teckidsProjectRetractionDeadline')

    cost = IntegerField(db_column='teckidsProjectCost')
    max_participants = IntegerField(db_column='teckidsProjectMaxParticipants')

    feedback_aspects = ListField(db_column='teckidsProjectFeedbackAspect')

    @property
    def can_register(self):
        now = date.today()

        if len(self.members) >= self.max_participants:
            return False

        if self.date_registration:
            return self.date_registration >= now
        else:
            return self.date_event > now

    @property
    def booked_percentage(self):
        return int(len(self.members) / self.max_participants * 100)

    def sync_members(self):
        # Ensure member_uids are in sync mith members
        self.member_uids = []
        for member_dn in self.members:
            # Rely on RDN being correctly set, for performance reasons
            rdn = member_dn.split(',')[0]
            rdn_parts = rdn.split('+')

            # Find the uid part of the RDN
            for rdn_part in rdn_parts:
                if rdn_part.startswith('uid='):
                    # Add to uids list and continue with next entry
                    uid = rdn_part.split('=')[1]
                    self.member_uids.append(uid)
                    break

    def add_member(self, person):
        # Add person both to groupOfNames and posixGroup part
        if not person.dn in self.members:
            self.members.append(person.dn)
        if not person.uid in self.member_uids:
            self.member_uids.append(person.uid)

    @property
    def members_persons(self):
        return self._get_objects('members', TeckidsPerson)

    @property
    def owners_persons(self):
        return self._get_objects('owners', TeckidsPerson)
